"""Serial random forest in sklearn."""
import argparse
import pathlib
from typing import Union
import time

import numpy as np
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split


def load_data_csv(
    path_to_data: Union[str, pathlib.Path],
    train_split: float = 0.75,
    seed: int = 42,
):
    """
    Load data from CSV file via root process.

    Params
    ------
    path_to_data : str
        The path to .csv file.
    train_split : float
        The train-test split fraction.
    seed : int
        The seed used for train-test split.

    Returns
    -------
    np.ndarray
        The train samples.
    np.ndarray
        The train targets.
    np.ndarray
        The test samples.
    nup.ndarray
        The test targets.
    """
    # Load data into numpy array.
    data = np.loadtxt(path_to_data, dtype=float, delimiter=",")
    # Divide data into samples and targets.
    samples = data[:, 1:]
    targets = data[:, 0]
    # Perform train-test split.
    samples_train, samples_test, targets_train, targets_test = train_test_split(
        samples, targets, test_size=1 - train_split, random_state=seed
    )
    return samples_train, targets_train, samples_test, targets_test


if __name__ == "__main__":
    print(
        "########################################\n"
        "# Serial Random Forest in Scikit-Learn #\n"
        "########################################\n"
    )
    parser = argparse.ArgumentParser(prog="Serial Random Forest")

    parser.add_argument(
        "--num_trees",
        type=int,
        default=100,
        help="The global number of trees.",
    )
    parser.add_argument(
        "--train_split",
        type=float,
        default=0.75,
        help="The fraction of the dataset used for training.",
    )
    parser.add_argument(
        "--seed",
        type=int,
        default=42,
        help="The random seed.",
    )
    args = parser.parse_args()
    path_to_data = "/pfs/work7/workspace/scratch/ku4408-VL-ScalableAI/data/SUSY.csv"
    print("Loading data...")
    start_load = time.perf_counter()
    (
        train_samples,
        train_targets,
        test_samples,
        test_targets,
    ) = load_data_csv(path_to_data, args.train_split, args.seed)
    elapsed_load = time.perf_counter() - start_load
    print(
        "DONE.\n"
        f"Train samples and targets have shapes {train_samples.shape} and {train_targets.shape}.\n"
        f"First ten elements are: {train_samples[:10]} and {train_targets[:10]}\n"
        f"Test samples and targets have shapes {test_samples.shape} and {test_targets.shape}.\n"
        f"First ten elements are: {test_samples[:10]} and {test_targets[:10]}\n"
        f"Time for data loading is {elapsed_load} s.\n"
        "Set up classifier."
    )
    classifier = RandomForestClassifier(
        n_estimators=args.num_trees, random_state=args.seed
    )
    start_train = time.perf_counter()
    print("Train.")
    _ = classifier.fit(train_samples, train_targets)
    acc = classifier.score(test_samples, test_targets)
    elapsed_train = time.perf_counter() - start_train
    print(f"Time for training is {elapsed_train} s.\n Accuracy is {acc}.")
